--
-- instr functions that mimic Oracle's counterpart
-- Syntax: instr(string1, string2, [n], [m]) where [] denotes optional parameters.
--
-- Searches string1 beginning at the nth character for the mth occurrence
-- of string2.  If n is negative, search backwards.  If m is not passed,
-- assume 1 (search starts at first character).
--

CREATE FUNCTION instr(varchar, varchar) RETURNS integer AS '
DECLARE
    pos integer;
BEGIN
    pos:= instr($1, $2, 1);
    RETURN pos;
END;
' LANGUAGE plpgsql;


CREATE FUNCTION instr(varchar, varchar, integer) RETURNS integer AS '
DECLARE
    string ALIAS FOR $1;
    string_to_search ALIAS FOR $2;
    beg_index ALIAS FOR $3;
    pos integer NOT NULL DEFAULT 0;
    temp_str varchar;
    beg integer;
    length integer;
    ss_length integer;
BEGIN
    IF beg_index > 0 THEN
        temp_str := substring(string FROM beg_index);
        pos := position(string_to_search IN temp_str);

        IF pos = 0 THEN
            RETURN 0;
        ELSE
            RETURN pos + beg_index - 1;
        END IF;
    ELSE
        ss_length := char_length(string_to_search);
        length := char_length(string);
        beg := length + beg_index - ss_length + 2;

        WHILE beg > 0 LOOP
            temp_str := substring(string FROM beg FOR ss_length);
            pos := position(string_to_search IN temp_str);

            IF pos > 0 THEN
                RETURN beg;
            END IF;

            beg := beg - 1;
        END LOOP;

        RETURN 0;
    END IF;
END;
' LANGUAGE plpgsql;



CREATE FUNCTION instr(varchar, varchar, integer, integer) RETURNS integer AS '
DECLARE
    string ALIAS FOR $1;
    string_to_search ALIAS FOR $2;
    beg_index ALIAS FOR $3;
    occur_index ALIAS FOR $4;
    pos integer NOT NULL DEFAULT 0;
    occur_number integer NOT NULL DEFAULT 0;
    temp_str varchar;
    beg integer;
    i integer;
    length integer;
    ss_length integer;
BEGIN
    IF beg_index > 0 THEN
        beg := beg_index;
        temp_str := substring(string FROM beg_index);

        FOR i IN 1..occur_index LOOP
            pos := position(string_to_search IN temp_str);

            IF i = 1 THEN
                beg := beg + pos - 1;
            ELSE
                beg := beg + pos;
            END IF;

            temp_str := substring(string FROM beg + 1);
        END LOOP;

        IF pos = 0 THEN
            RETURN 0;
        ELSE
            RETURN beg;
        END IF;
    ELSE
        ss_length := char_length(string_to_search);
        length := char_length(string);
        beg := length + beg_index - ss_length + 2;

        WHILE beg > 0 LOOP
            temp_str := substring(string FROM beg FOR ss_length);
            pos := position(string_to_search IN temp_str);

            IF pos > 0 THEN
                occur_number := occur_number + 1;

                IF occur_number = occur_index THEN
                    RETURN beg;
                END IF;
            END IF;

            beg := beg - 1;
        END LOOP;

        RETURN 0;
    END IF;
END;
' LANGUAGE plpgsql;

--
-- Tests whether an entry is accessible according to its ACLs
--
CREATE OR REPLACE FUNCTION acl_test(varchar, varchar, varchar) RETURNS integer AS '
DECLARE
    acls ALIAS FOR $1;
    groups ALIAS FOR $2;
    right ALIAS FOR $3;
    group varchar;
    pos1 integer;
    pos2 integer;
    groupstart integer;
BEGIN
    IF acls IS NULL THEN
	RETURN NULL;
    END IF;

--    RAISE NOTICE ''ACLS are >%<'', acls;
    pos1 := 1;
    LOOP
        group := NULL;

        pos2 := instr(groups,'' '', pos1 + 1);
        IF pos2 = 0 THEN
	    pos2 = LENGTH(groups) + 1;
        END IF;

	EXIT WHEN pos1 = pos2;

        group := substr(groups, pos1, pos2 - pos1);
--	RAISE NOTICE ''Group is >%<'', group;

	groupstart := instr(acls, ''|'' || group || '' '');
	IF groupstart != 0 THEN
	    groupstart := groupstart + LENGTH(group) + 2;
	    LOOP
--		RAISE NOTICE ''CHAR >%<'', substr(acls, groupstart, 1);
		EXIT WHEN substr(acls, groupstart, 1) = ''|'';
		IF substr(acls, groupstart, 1) = right THEN
		    RETURN 1;
		END IF;
		groupstart = groupstart + 1;
	    END LOOP;
	END IF;

        pos1 := pos2 + 1;
	EXIT WHEN pos1 >= LENGTH(groups);

    END LOOP;
    RETURN NULL;
END;
' LANGUAGE plpgsql;
